@@ -25,6 +25,7 @@
 #include "full-screen.h"
 #include "image-view.h"
 #include "ui-image.h"
+#include "gconf.h"
 
 /* Private part of the FullScreen structure */
 struct _FullScreenPrivate {
@@ -114,8 +115,7 @@ full_screen_class_init (FullScreenClass *class)
 
 /* Handler for changes on the sb policy */
 static void
-sb_policy_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		      GConfValue *value, gboolean is_default, gpointer data)
+sb_policy_changed_cb (GConfClient *client, guint notify_id, GConfEntry *entry, gpointer data)
 {
 	FullScreen *fs;
 	FullScreenPrivate *priv;
@@ -123,15 +123,14 @@ sb_policy_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
 	fs = FULL_SCREEN (data);
 	priv = fs->priv;
 
-	priv->sb_policy = gconf_value_int (value);
+	priv->sb_policy = gconf_value_get_int (entry->value);
 
 	gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);
 }
 
 /* Handler for changes on the bevel policy */
 static void
-bevel_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		  GConfValue *value, gboolean is_default, gpointer data)
+bevel_changed_cb (GConfClient *client, guint notify_id, GConfEntry *entry, gpointer data)
 {
 	FullScreen *fs;
 	FullScreenPrivate *priv;
@@ -139,7 +138,7 @@ bevel_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
 	fs = FULL_SCREEN (data);
 	priv = fs->priv;
 
-	priv->bevel = gconf_value_bool (value);
+	priv->bevel = gconf_value_get_bool (entry->value);
 
 	gtk_scroll_frame_set_shadow_type (GTK_SCROLL_FRAME (priv->ui),
 					  priv->bevel ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
@@ -150,6 +149,7 @@ static void
 full_screen_init (FullScreen *fs)
 {
 	FullScreenPrivate *priv;
+	ImageView *view;
 
 	priv = g_new0 (FullScreenPrivate, 1);
 	fs->priv = priv;
@@ -180,6 +180,9 @@ full_screen_init (FullScreen *fs)
 	gtk_widget_set_uposition (GTK_WIDGET (fs), 0, 0);
 
 	priv->ui = ui_image_new ();
+	view = IMAGE_VIEW (ui_image_get_image_view (UI_IMAGE (priv->ui)));
+	image_view_add_gconf_client (view, priv->client);
+
 	gtk_container_add (GTK_CONTAINER (fs), priv->ui);
 
 	gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);
