@@ -25,7 +25,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
-#include <gconf/gconf-client.h>
 #include "cursors.h"
 #include "image-view.h"
 #include "uta.h"
@@ -43,11 +42,11 @@
 
 /* Maximum size of repaint rectangles */
 
-#define PAINT_UTA1_RECT_WIDTH 128
-#define PAINT_UTA1_RECT_HEIGHT 128
+#define PAINT_FAST_RECT_WIDTH 256
+#define PAINT_FAST_RECT_HEIGHT 256
 
-#define PAINT_UTA2_RECT_WIDTH 128
-#define PAINT_UTA2_RECT_HEIGHT 128
+#define PAINT_INTERP_RECT_WIDTH 128
+#define PAINT_INTERP_RECT_HEIGHT 128
 
 /* Scroll step increment */
 
@@ -62,11 +61,13 @@ struct _ImageViewPrivate {
 	/* Image being displayed */
 	Image *image;
 
-	/* Current zoom factor */
-	double zoom;
+	/* Current zoom factors */
+	double zoomx;
+	double zoomy;
 
 	/* Previous zoom factor and zoom anchor point stored for size_allocate */
-	double old_zoom;
+	double old_zoomx;
+	double old_zoomy;
 	double zoom_x_anchor;
 	double zoom_y_anchor;
 
@@ -106,16 +107,6 @@ struct _ImageViewPrivate {
 	/* Scroll type */
 	ScrollType scroll;
 
-	/* GConf client for monitoring changes to the preferences */
-	GConfClient *client;
-
-	/* GConf client notify IDs */
-	guint interp_type_notify_id;
-	guint check_type_notify_id;
-	guint check_size_notify_id;
-	guint dither_notify_id;
-	guint scroll_notify_id;
-
 	/* Whether the image is being dragged */
 	guint dragging : 1;
 
@@ -129,6 +120,17 @@ enum {
 	LAST_SIGNAL
 };
 
+enum {
+	ARG_0,
+	ARG_INTERP_TYPE,
+	ARG_CHECK_TYPE,
+	ARG_CHECK_SIZE,
+	ARG_DITHER,
+	ARG_SCROLL,
+	ARG_FULL_SCREEN_ZOOM
+};
+
+
 static void image_view_class_init (ImageViewClass *class);
 static void image_view_init (ImageView *view);
 static void image_view_destroy (GtkObject *object);
@@ -145,6 +147,9 @@ static gint image_view_button_release (GtkWidget *widget, GdkEventButton *event)
 static gint image_view_motion (GtkWidget *widget, GdkEventMotion *event);
 static gint image_view_expose (GtkWidget *widget, GdkEventExpose *event);
 static gint image_view_key_press (GtkWidget *widget, GdkEventKey *event);
+static void image_view_get_arg (GtkObject* obj, GtkArg* arg, guint arg_id);
+static void image_view_set_arg (GtkObject* obj, GtkArg* arg, guint arg_id);
+
 
 static void image_view_set_scroll_adjustments (GtkWidget *widget,
 					       GtkAdjustment *hadj, GtkAdjustment *vadj);
@@ -199,6 +204,13 @@ image_view_class_init (ImageViewClass *class)
 
 	parent_class = gtk_type_class (GTK_TYPE_WIDGET);
 
+	gtk_object_add_arg_type ("ImageView::interp_type", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_INTERP_TYPE);
+	gtk_object_add_arg_type ("ImageView::check_type", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_CHECK_TYPE);
+	gtk_object_add_arg_type ("ImageView::check_size", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_CHECK_SIZE);
+	gtk_object_add_arg_type ("ImageView::dither", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_DITHER);
+	gtk_object_add_arg_type ("ImageView::scroll", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_SCROLL);
+	gtk_object_add_arg_type ("ImageView::full_screen_zoom", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_FULL_SCREEN_ZOOM);
+
 	image_view_signals[ZOOM_FIT] =
 		gtk_signal_new ("zoom_fit",
 				GTK_RUN_FIRST,
@@ -234,46 +246,70 @@ image_view_class_init (ImageViewClass *class)
 	widget_class->motion_notify_event = image_view_motion;
 	widget_class->expose_event = image_view_expose;
 	widget_class->key_press_event = image_view_key_press;
+
+	object_class->get_arg = image_view_get_arg;
+	object_class->set_arg = image_view_set_arg;
 }
 
-/* Handler for changes on the interp_type value */
 static void
-interp_type_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-			GConfValue *value, gboolean is_default, gpointer data)
+image_view_get_arg (GtkObject* obj, GtkArg* arg, guint arg_id)
 {
-	image_view_set_interp_type (IMAGE_VIEW (data), gconf_value_int (value));
+	ImageView *image_view = IMAGE_VIEW (obj);
+	ImageViewPrivate *priv = image_view->priv;
+
+	switch (arg_id) {
+	case ARG_INTERP_TYPE:
+		GTK_VALUE_INT(*arg) = priv->interp_type;
+		break;
+	case ARG_CHECK_TYPE:
+		GTK_VALUE_INT(*arg) = priv->check_type;
+		break;
+	case ARG_CHECK_SIZE:
+		GTK_VALUE_INT(*arg) = priv->check_size;
+		break;
+	case ARG_DITHER:
+		GTK_VALUE_INT(*arg) = priv->dither;
+		break;
+	case ARG_SCROLL:
+		GTK_VALUE_INT(*arg) = priv->scroll;
+		break;
+	case ARG_FULL_SCREEN_ZOOM:
+		GTK_VALUE_INT(*arg) = priv->full_screen_zoom;
+		break;
+	default:
+		g_warning ("unknown arg id `%d'", arg_id);
+		break;
+	}
 }
 
-/* Handler for changes on the check_type value */
 static void
-check_type_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		       GConfValue *value, gboolean is_default, gpointer data)
+image_view_set_arg (GtkObject* obj, GtkArg* arg, guint arg_id)
 {
-	image_view_set_check_type (IMAGE_VIEW (data), gconf_value_int (value));
-}
+	ImageView *image_view = IMAGE_VIEW (obj);
 
-/* Handler for changes on the check_size value */
-static void
-check_size_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		       GConfValue *value, gboolean is_default, gpointer data)
-{
-	image_view_set_check_size (IMAGE_VIEW (data), gconf_value_int (value));
-}
-
-/* Handler for changes on the dither value */
-static void
-dither_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		   GConfValue *value, gboolean is_default, gpointer data)
-{
-	image_view_set_dither (IMAGE_VIEW (data), gconf_value_int (value));
-}
-
-/* Handler for changes on the scroll value */
-static void
-scroll_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		   GConfValue *value, gboolean is_default, gpointer data)
-{
-	image_view_set_scroll (IMAGE_VIEW (data), gconf_value_int (value));
+	switch (arg_id) {
+	case ARG_INTERP_TYPE:
+		image_view_set_interp_type (image_view, GTK_VALUE_INT(*arg));
+		break;
+	case ARG_CHECK_TYPE:
+		image_view_set_check_type (image_view, GTK_VALUE_INT(*arg));
+		break;
+	case ARG_CHECK_SIZE:
+		image_view_set_check_size (image_view, GTK_VALUE_INT(*arg));
+		break;
+	case ARG_DITHER:
+		image_view_set_dither (image_view, GTK_VALUE_INT(*arg));
+		break;
+	case ARG_SCROLL:
+		image_view_set_scroll (image_view, GTK_VALUE_INT(*arg));
+		break;
+	case ARG_FULL_SCREEN_ZOOM:
+		image_view_set_full_screen_zoom (image_view, GTK_VALUE_INT(*arg));
+		break;
+	default:
+		g_warning ("unknown arg id `%d'", arg_id);
+		break;
+	}
 }
 
 /* Object initialization function for the image view */
@@ -288,56 +324,7 @@ image_view_init (ImageView *view)
 	GTK_WIDGET_UNSET_FLAGS (view, GTK_NO_WINDOW);
 	GTK_WIDGET_SET_FLAGS (view, GTK_CAN_FOCUS);
 
-	priv->zoom = 1.0;
-
-	/* Add the GConf client and notification handlers */
-
-	priv->client = gconf_client_get_default ();
-
-	gconf_client_add_dir (priv->client, "/apps/eog",
-			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-
-	priv->interp_type_notify_id = gconf_client_notify_add (
-		priv->client, "/apps/eog/view/interp_type",
-		interp_type_changed_cb, view,
-		NULL, NULL);
-	priv->check_type_notify_id = gconf_client_notify_add (
-		priv->client, "/apps/eog/view/check_type",
-		check_type_changed_cb, view,
-		NULL, NULL);
-	priv->check_size_notify_id = gconf_client_notify_add (
-		priv->client, "/apps/eog/view/check_size",
-		check_size_changed_cb, view,
-		NULL, NULL);
-	priv->dither_notify_id = gconf_client_notify_add (
-		priv->client, "/apps/eog/view/dither",
-		dither_changed_cb, view,
-		NULL, NULL);
-	priv->scroll_notify_id = gconf_client_notify_add (
-		priv->client, "/apps/eog/view/scroll",
-		scroll_changed_cb, view,
-		NULL, NULL);
-
-	/* Get the default values */
-
-	priv->interp_type = gconf_client_get_int (
-		priv->client, "/apps/eog/view/interp_type",
-		NULL);
-	priv->check_type = gconf_client_get_int (
-		priv->client, "/apps/eog/view/check_type",
-		NULL);
-	priv->check_size = gconf_client_get_int (
-		priv->client, "/apps/eog/view/check_size",
-		NULL);
-	priv->dither = gconf_client_get_int (
-		priv->client, "/apps/eog/view/dither",
-		NULL);
-	priv->scroll = gconf_client_get_int (
-		priv->client, "/apps/eog/view/scroll",
-		NULL);
-	priv->full_screen_zoom = gconf_client_get_int (
-		priv->client, "/apps/eog/full_screen/zoom",
-		NULL);
+	priv->zoomx = priv->zoomy = 1.0;
 }
 
 /* Frees the dirty region uta and removes the idle handler */
@@ -383,22 +370,6 @@ image_view_destroy (GtkObject *object)
 	gtk_signal_disconnect_by_data (GTK_OBJECT (priv->hadj), view);
 	gtk_signal_disconnect_by_data (GTK_OBJECT (priv->vadj), view);
 
-	/* Remove notification handlers */
-
-	gconf_client_notify_remove (priv->client, priv->interp_type_notify_id);
-	gconf_client_notify_remove (priv->client, priv->check_type_notify_id);
-	gconf_client_notify_remove (priv->client, priv->check_size_notify_id);
-	gconf_client_notify_remove (priv->client, priv->dither_notify_id);
-	gconf_client_notify_remove (priv->client, priv->scroll_notify_id);
-
-	priv->interp_type_notify_id = 0;
-	priv->check_type_notify_id = 0;
-	priv->check_size_notify_id = 0;
-	priv->dither_notify_id = 0;
-	priv->scroll_notify_id = 0;
-
-	gconf_client_remove_dir (priv->client, "/apps/eog", NULL);
-
 	/* Clean up */
 
 	remove_dirty_region (view);
@@ -431,9 +402,6 @@ image_view_finalize (GtkObject *object)
 	gtk_object_unref (GTK_OBJECT (priv->vadj));
 	priv->vadj = NULL;
 
-	gtk_object_unref (GTK_OBJECT (priv->client));
-	priv->client = NULL;
-
 	g_free (priv);
 	view->priv = NULL;
 
@@ -447,15 +415,15 @@ image_view_finalize (GtkObject *object)
 
 /* Computes the size in pixels of the scaled image */
 static void
-compute_scaled_size (ImageView *view, double zoom, int *width, int *height)
+compute_scaled_size (ImageView *view, double zoomx, double zoomy, int *width, int *height)
 {
 	ImageViewPrivate *priv;
 
 	priv = view->priv;
 
 	if (priv->image && priv->image->pixbuf) {
-		*width = floor (gdk_pixbuf_get_width (priv->image->pixbuf) * zoom + 0.5);
-		*height = floor (gdk_pixbuf_get_height (priv->image->pixbuf) * zoom + 0.5);
+		*width = floor (gdk_pixbuf_get_width (priv->image->pixbuf) * zoomx + 0.5);
+		*height = floor (gdk_pixbuf_get_height (priv->image->pixbuf) * zoomy + 0.5);
 	} else
 		*width = *height = 0;
 }
@@ -532,6 +500,14 @@ pack_pixbuf (GdkPixbuf *pixbuf)
 
 #endif
 
+#define DOUBLE_EQUAL(a,b) (fabs (a - b) < 1e-6)
+static gboolean
+unity_zoom (ImageViewPrivate *priv)
+{
+	return (DOUBLE_EQUAL (priv->zoomx, 1.0) &&
+		DOUBLE_EQUAL (priv->zoomx, 1.0));
+}
+
 /* Paints a rectangle of the dirty region */
 static void
 paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
@@ -547,7 +523,7 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 
 	priv = view->priv;
 
-	compute_scaled_size (view, priv->zoom, &scaled_width, &scaled_height);
+	compute_scaled_size (view, priv->zoomx, priv->zoomy, &scaled_width, &scaled_height);
 
 	width = GTK_WIDGET (view)->allocation.width;
 	height = GTK_WIDGET (view)->allocation.height;
@@ -623,7 +599,8 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 
 	/* Short-circuit the fast case to avoid a memcpy() */
 
-	if (priv->zoom == 1.0
+
+	if (unity_zoom (priv)
 	    && gdk_pixbuf_get_colorspace (priv->image->pixbuf) == GDK_COLORSPACE_RGB
 	    && !gdk_pixbuf_get_has_alpha (priv->image->pixbuf)
 	    && gdk_pixbuf_get_bits_per_sample (priv->image->pixbuf) == 8) {
@@ -721,8 +698,8 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 				    0, 0,
 				    d.x1 - d.x0, d.y1 - d.y0,
 				    -(d.x0 - xofs), -(d.y0 - yofs),
-				    priv->zoom, priv->zoom,
-				    (priv->zoom == 1.0) ? GDK_INTERP_NEAREST : interp_type,
+				    priv->zoomx, priv->zoomy,
+				    unity_zoom (priv) ? GDK_INTERP_NEAREST : interp_type,
 				    255,
 				    d.x0 - xofs, d.y0 - yofs,
 				    check_size,
@@ -755,6 +732,8 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 #endif
 }
 
+#include <stdio.h>
+
 /* Idle handler for the drawing process.  We pull a rectangle from the dirty
  * region microtile array, paint it, and leave the rest to the next idle
  * iteration.
@@ -772,28 +751,39 @@ paint_iteration_idle (gpointer data)
 	g_assert (priv->uta1 != NULL || priv->uta2 != NULL);
 
 	if (priv->uta1) {
-		pull_rectangle (priv->uta1, &rect, PAINT_UTA1_RECT_WIDTH, PAINT_UTA1_RECT_HEIGHT);
+		/* Paint the no-interpolation cases as fast as possible */
 
-		if (art_irect_empty (&rect)) {
-			art_uta_free (priv->uta1);
-			priv->uta1 = NULL;
-			goto uta2;
+		if (priv->scroll == SCROLL_TWO_PASS || priv->interp_type == GDK_INTERP_NEAREST)
+			while (1) {
+				pull_rectangle (priv->uta1, &rect,
+						PAINT_FAST_RECT_WIDTH, PAINT_FAST_RECT_HEIGHT);
+
+				if (art_irect_empty (&rect)) {
+					art_uta_free (priv->uta1);
+					priv->uta1 = NULL;
+					break;
+				}
+
+				paint_rectangle (view, &rect, GDK_INTERP_NEAREST);
+
+				if (priv->scroll == SCROLL_TWO_PASS)
+					priv->uta2 = uta_add_rect (priv->uta2,
+								   rect.x0, rect.y0,
+								   rect.x1, rect.y1);
+			}
+		else {
+			pull_rectangle (priv->uta1, &rect,
+					PAINT_INTERP_RECT_WIDTH, PAINT_INTERP_RECT_HEIGHT);
+
+			if (art_irect_empty (&rect)) {
+				art_uta_free (priv->uta1);
+				priv->uta1 = NULL;
+			} else
+				paint_rectangle (view, &rect, priv->interp_type);
 		}
-
-		if (priv->scroll == SCROLL_TWO_PASS) {
-			paint_rectangle (view, &rect, GDK_INTERP_NEAREST);
-
-			priv->uta2 = uta_add_rect (priv->uta2, rect.x0, rect.y0, rect.x1, rect.y1);
-		} else
-			paint_rectangle (view, &rect, priv->interp_type);
-
-		goto out;
-	}
-
- uta2:
-
-	if (priv->uta2) {
-		pull_rectangle (priv->uta2, &rect, PAINT_UTA2_RECT_WIDTH, PAINT_UTA2_RECT_HEIGHT);
+	} else if (priv->uta2) {
+		pull_rectangle (priv->uta2, &rect,
+				PAINT_INTERP_RECT_WIDTH, PAINT_INTERP_RECT_HEIGHT);
 
 		if (art_irect_empty (&rect)) {
 			art_uta_free (priv->uta2);
@@ -802,8 +792,6 @@ paint_iteration_idle (gpointer data)
 			paint_rectangle (view, &rect, priv->interp_type);
 	}
 
- out:
-
 	if (!priv->uta1 && !priv->uta2) {
 		priv->idle_id = 0;
 		return FALSE;
@@ -832,9 +820,11 @@ request_paint_area (ImageView *view, GdkRectangle *area, gboolean asynch)
 	if (r.x0 >= r.x1 || r.y0 >= r.y1)
 		return;
 
-	/* Do nearest neighbor or 1:1 zoom synchronously for speed */
+	/* Unless told not to, do nearest neighbor or 1:1 zoom synchronously for
+         * speed.
+	 */
 
-	if (priv->interp_type == GDK_INTERP_NEAREST || priv->zoom == 1.0) {
+	if (!asynch && (priv->interp_type == GDK_INTERP_NEAREST || unity_zoom (priv))) {
 		paint_rectangle (view, &r, priv->interp_type);
 		return;
 	}
@@ -1112,29 +1102,29 @@ compute_center_zoom_offsets (ImageView *view,
 	priv = view->priv;
 	g_assert (priv->need_zoom_change);
 
-	compute_scaled_size (view, priv->old_zoom, &old_scaled_width, &old_scaled_height);
+	compute_scaled_size (view, priv->old_zoomx, priv->old_zoomy, &old_scaled_width, &old_scaled_height);
 
 	if (old_scaled_width < old_width)
-		view_cx = (priv->zoom_x_anchor * old_scaled_width) / priv->old_zoom;
+		view_cx = (priv->zoom_x_anchor * old_scaled_width) / priv->old_zoomx;
 	else
-		view_cx = (priv->xofs + priv->zoom_x_anchor * old_width) / priv->old_zoom;
+		view_cx = (priv->xofs + priv->zoom_x_anchor * old_width) / priv->old_zoomx;
 
 	if (old_scaled_height < old_height)
-		view_cy = (priv->zoom_y_anchor * old_scaled_height) / priv->old_zoom;
+		view_cy = (priv->zoom_y_anchor * old_scaled_height) / priv->old_zoomy;
 	else
-		view_cy = (priv->yofs + priv->zoom_y_anchor * old_height) / priv->old_zoom;
+		view_cy = (priv->yofs + priv->zoom_y_anchor * old_height) / priv->old_zoomy;
 
-	compute_scaled_size (view, priv->zoom, &new_scaled_width, &new_scaled_height);
+	compute_scaled_size (view, priv->zoomx, priv->zoomy, &new_scaled_width, &new_scaled_height);
 
 	if (new_scaled_width < new_width)
 		*xofs = 0;
 	else
-		*xofs = floor (view_cx * priv->zoom - priv->zoom_x_anchor * new_width + 0.5);
+		*xofs = floor (view_cx * priv->zoomx - priv->zoom_x_anchor * new_width + 0.5);
 
 	if (new_scaled_height < new_height)
 		*yofs = 0;
 	else
-		*yofs = floor (view_cy * priv->zoom - priv->zoom_y_anchor * new_height + 0.5);
+		*yofs = floor (view_cy * priv->zoomy - priv->zoom_y_anchor * new_height + 0.5);
 }
 
 /* Size_allocate handler for the image view */
@@ -1181,7 +1171,7 @@ image_view_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
 	/* Set scroll increments */
 
-	compute_scaled_size (view, priv->zoom, &scaled_width, &scaled_height);
+	compute_scaled_size (view, priv->zoomx, priv->zoomy, &scaled_width, &scaled_height);
 
 	priv->hadj->page_size = MIN (scaled_width, allocation->width);
 	priv->hadj->page_increment = allocation->width / 2;
@@ -1283,12 +1273,12 @@ image_view_button_press (GtkWidget *widget, GdkEventButton *event)
 
 	case 4:
 		set_zoom_anchor (view, event->x, event->y);
-		image_view_set_zoom (view, priv->zoom * 1.05);
+		image_view_set_zoom (view, priv->zoomx * 1.05, priv->zoomy * 1.05);
 		return TRUE;
 
 	case 5:
 		set_zoom_anchor (view, event->x, event->y);
-		image_view_set_zoom (view, priv->zoom / 1.05);
+		image_view_set_zoom (view, priv->zoomx / 1.05, priv->zoomy / 1.05);
 		return TRUE;
 
 	default:
@@ -1397,7 +1387,7 @@ image_view_key_press (GtkWidget *widget, GdkEventKey *event)
 	ImageView *view;
 	ImageViewPrivate *priv;
 	gboolean do_zoom;
-	double zoom;
+	double zoomx, zoomy;
 	gboolean do_scroll;
 	int xofs, yofs;
 
@@ -1407,7 +1397,7 @@ image_view_key_press (GtkWidget *widget, GdkEventKey *event)
 	do_zoom = FALSE;
 	do_scroll = FALSE;
 	xofs = yofs = 0;
-	zoom = 1.0;
+	zoomx = zoomy = 1.0;
 
 	if ((event->state & (GDK_MODIFIER_MASK & ~GDK_LOCK_MASK)) != 0)
 		return FALSE;
@@ -1440,18 +1430,20 @@ image_view_key_press (GtkWidget *widget, GdkEventKey *event)
 	case GDK_equal:
 	case GDK_KP_Add:
 		do_zoom = TRUE;
-		zoom = priv->zoom * 1.05;
+		zoomx = priv->zoomx * 1.05;
+		zoomy = priv->zoomy * 1.05;
 		break;
 
 	case GDK_minus:
 	case GDK_KP_Subtract:
 		do_zoom = TRUE;
-		zoom = priv->zoom / 1.05;
+		zoomx = priv->zoomx / 1.05;
+		zoomy = priv->zoomy / 1.05;
 		break;
 
 	case GDK_1:
 		do_zoom = TRUE;
-		zoom = 1.0;
+		zoomx = zoomy = 1.0;
 		break;
 
 	case GDK_F:
@@ -1474,7 +1466,7 @@ image_view_key_press (GtkWidget *widget, GdkEventKey *event)
 		else
 			set_default_zoom_anchor (view);
 
-		image_view_set_zoom (view, zoom);
+		image_view_set_zoom (view, zoomx, zoomy);
 	}
 
 	if (do_scroll) {
@@ -1517,7 +1509,8 @@ adjustment_changed_cb (GtkAdjustment *adj, gpointer data)
 /* Set_scroll_adjustments handler for the image view */
 static void
 image_view_set_scroll_adjustments (GtkWidget *widget,
-				   GtkAdjustment *hadj, GtkAdjustment *vadj)
+				   GtkAdjustment *hadj,
+				   GtkAdjustment *vadj)
 {
 	ImageView *view;
 	ImageViewPrivate *priv;
@@ -1595,6 +1588,20 @@ image_view_new (void)
 	return GTK_WIDGET (gtk_type_new (TYPE_IMAGE_VIEW));
 }
 
+/* Requests a full redraw of the image view */
+static void
+redraw_all (ImageView *view)
+{
+	GdkRectangle r;
+
+	r.x = 0;
+	r.y = 0;
+	r.width = GTK_WIDGET (view)->allocation.width;
+	r.height = GTK_WIDGET (view)->allocation.height;
+
+	request_paint_area (view, &r, TRUE);
+}
+
 /**
  * image_view_set_image:
  * @view: An image view.
@@ -1625,7 +1632,7 @@ image_view_set_image (ImageView *view, Image *image)
 
 	/* FIXME: adjust zoom / image offsets; maybe just offsets here */
 
-	gtk_widget_queue_resize (GTK_WIDGET (view));
+	redraw_all (view);
 }
 
 /**
@@ -1656,28 +1663,34 @@ image_view_get_image (ImageView *view)
  * Sets the zoom factor for an image view.
  **/
 void
-image_view_set_zoom (ImageView *view, double zoom)
+image_view_set_zoom (ImageView *view, double zoomx, double zoomy)
 {
 	ImageViewPrivate *priv;
 
 	g_return_if_fail (view != NULL);
 	g_return_if_fail (IS_IMAGE_VIEW (view));
-	g_return_if_fail (zoom > 0.0);
+	g_return_if_fail (zoomx > 0.0);
+	g_return_if_fail (zoomy > 0.0);
 
 	priv = view->priv;
 
-	if (zoom > MAX_ZOOM_FACTOR)
-		zoom = MAX_ZOOM_FACTOR;
+	if (zoomx > MAX_ZOOM_FACTOR)
+		zoomx = MAX_ZOOM_FACTOR;
+	if (zoomy > MAX_ZOOM_FACTOR)
+		zoomy = MAX_ZOOM_FACTOR;
 
-	if (priv->zoom == zoom)
+	if (DOUBLE_EQUAL (priv->zoomx, zoomx) &&
+	    DOUBLE_EQUAL (priv->zoomy, zoomy))
 		return;
 
 	if (!priv->need_zoom_change) {
-		priv->old_zoom = priv->zoom;
+		priv->old_zoomx = priv->zoomx;
+		priv->old_zoomy = priv->zoomy;
 		priv->need_zoom_change = TRUE;
 	}
 
-	priv->zoom = zoom;
+	priv->zoomx = zoomx;
+	priv->zoomy = zoomy;
 
 	gtk_widget_queue_resize (GTK_WIDGET (view));
 }
@@ -1699,7 +1712,7 @@ image_view_get_zoom (ImageView *view)
 	g_return_val_if_fail (IS_IMAGE_VIEW (view), -1.0);
 
 	priv = view->priv;
-	return priv->zoom;
+	return (priv->zoomx + priv->zoomy) / 2;
 }
 
 /**
@@ -1723,7 +1736,7 @@ image_view_set_interp_type (ImageView *view, GdkInterpType interp_type)
 		return;
 
 	priv->interp_type = interp_type;
-	gtk_widget_queue_draw (GTK_WIDGET (view));
+	redraw_all (view);
 }
 
 /**
@@ -1767,7 +1780,7 @@ image_view_set_check_type (ImageView *view, CheckType check_type)
 		return;
 
 	priv->check_type = check_type;
-	gtk_widget_queue_draw (GTK_WIDGET (view));
+	redraw_all (view);
 }
 
 /**
@@ -1811,7 +1824,7 @@ image_view_set_check_size (ImageView *view, CheckSize check_size)
 		return;
 
 	priv->check_size = check_size;
-	gtk_widget_queue_draw (GTK_WIDGET (view));
+	redraw_all (view);
 }
 
 /**
@@ -1855,7 +1868,7 @@ image_view_set_dither (ImageView *view, GdkRgbDither dither)
 		return;
 
 	priv->dither = dither;
-	gtk_widget_queue_draw (GTK_WIDGET (view));
+	redraw_all (view);
 }
 
 /**
@@ -1899,7 +1912,7 @@ image_view_set_scroll (ImageView *view, ScrollType scroll)
 		return;
 
 	priv->scroll = scroll;
-	gtk_widget_queue_draw (GTK_WIDGET (view));
+	redraw_all (view);
 }
 
 /**
@@ -1921,3 +1934,47 @@ image_view_get_scroll (ImageView *view)
 	priv = view->priv;
 	return priv->scroll;
 }
+
+/**
+ * image_view_set_full_screen_zoom:
+ * @view: An image view.
+ * @full_screen_zoom: Full screen zooming type.
+ *
+ * Sets the full screen zooming type on an image view.
+ **/
+void
+image_view_set_full_screen_zoom (ImageView *view, FullScreenZoom full_screen_zoom)
+{
+	ImageViewPrivate *priv;
+
+	g_return_if_fail (view != NULL);
+	g_return_if_fail (IS_IMAGE_VIEW (view));
+
+	priv = view->priv;
+
+	if (priv->full_screen_zoom == full_screen_zoom)
+		return;
+
+	priv->full_screen_zoom = full_screen_zoom;
+	redraw_all (view);
+}
+
+/**
+ * image_view_get_full_screen_zoom:
+ * @view: An image view.
+ *
+ * Queries the full_screen_zooming type of an image view.
+ *
+ * Return value: full screen zooming type.
+ **/
+FullScreenZoom
+image_view_get_full_screen_zoom (ImageView *view)
+{
+	ImageViewPrivate *priv;
+
+	g_return_val_if_fail (view != NULL, FULL_SCREEN_ZOOM_1);
+	g_return_val_if_fail (IS_IMAGE_VIEW (view), FULL_SCREEN_ZOOM_1);
+
+	priv = view->priv;
+	return priv->full_screen_zoom;
+}
