@@ -31,6 +31,7 @@
 #include "util.h"
 #include "window.h"
 #include "zoom.h"
+#include "gconf.h"
 
 /* Default size for windows */
 
@@ -152,7 +153,7 @@ about_cmd (GtkWidget *widget, gpointer data)
 	static GtkWidget *about;
 	static const char *authors[] = {
 		"Federico Mena-Quintero (federico@gnu.org)",
-		"Arik Devens (arik@helixcode.com)",
+		"Arik Devens (arik@gnome.org)",
 		NULL
 	};
 
@@ -160,7 +161,7 @@ about_cmd (GtkWidget *widget, gpointer data)
 		about = gnome_about_new (
 			_("Eye of Gnome"),
 			VERSION,
-			_("Copyright (C) 2000 The Free Software Foundation"),
+			_("Copyright (C) 2000-2001 The Free Software Foundation"),
 			authors,
 			_("The GNOME image viewing and cataloging program"),
 			NULL);
@@ -313,8 +314,7 @@ window_class_init (WindowClass *class)
 
 /* Handler for changes on the window sb policy */
 static void
-sb_policy_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
-		      GConfValue *value, gboolean is_default, gpointer data)
+sb_policy_changed_cb (GConfClient *client, guint notify_id, GConfEntry *entry, gpointer data)
 {
 	Window *window;
 	WindowPrivate *priv;
@@ -322,7 +322,7 @@ sb_policy_changed_cb (GConfClient *client, guint notify_id, const gchar *key,
 	window = WINDOW (data);
 	priv = window->priv;
 
-	priv->sb_policy = gconf_value_int (value);
+	priv->sb_policy = gconf_value_get_int (entry->value);
 
 	gtk_scroll_frame_set_policy (GTK_SCROLL_FRAME (priv->ui), priv->sb_policy, priv->sb_policy);
 }
@@ -551,6 +551,7 @@ set_drag_dest (Window *window)
 			   GDK_ACTION_COPY);
 }
 
+
 /**
  * window_construct:
  * @window: A window widget.
@@ -583,6 +584,7 @@ window_construct (Window *window)
 	gtk_widget_show (priv->ui);
 
 	view = IMAGE_VIEW (ui_image_get_image_view (UI_IMAGE (priv->ui)));
+	image_view_add_gconf_client (view, priv->client);
 
 	gtk_widget_grab_focus (GTK_WIDGET (view));
 
@@ -756,7 +758,7 @@ auto_size (Window *window)
 	sheight = gdk_screen_height () * 0.75;
 
 	zoom = zoom_fit_scale (swidth, sheight, iwidth, iheight, FALSE);
-	image_view_set_zoom (IMAGE_VIEW (view), zoom);
+	image_view_set_zoom (IMAGE_VIEW (view), zoom, zoom);
 
 	zwidth = floor (iwidth * zoom + 0.5);
 	zheight = floor (iheight * zoom + 0.5);
